/* INCLUDES ************************************************/
#include "spi_driver.h"
#include "nrf_delay.h"
#include "nrf_log.h"
#include "nrf_gpio.h"
#include "TRI_SSM_REV_A.h"

#include "nrf_delay.h"

#define SPI_INSTANCE  1 /**< SPI instance index. */
static const nrfx_spim_t m_spi = NRFX_SPIM_INSTANCE(SPI_INSTANCE);    /**< SPI instance declaration */
static volatile bool spi_xfer_done;  /**< Flag used to indicate that SPI instance completed the transfer. */

#define TEST_STRING "Nordic"
static uint8_t       m_tx_buf[] = TEST_STRING;           /**< SPI TX buffer. */
static uint8_t       m_rx_buf[sizeof(TEST_STRING) + 1];  /**< SPI RX buffer. */
static const uint8_t m_length = sizeof(m_tx_buf);        /**< SPI Transfer length. */

/**
 * @brief SPI user event handler.
 * @param event
 */
void spim_event_handler(nrfx_spim_evt_t const * p_event,
                       void *                    p_context)
{
    spi_xfer_done = true;
    //NRF_LOG_INFO("Transfer completed.");
    if (m_rx_buf[0] != 0)
    {
        NRF_LOG_INFO(" Received:");
        NRF_LOG_HEXDUMP_INFO(m_rx_buf, strlen((const char *)m_rx_buf));
    }
}

/**
 * @brief Global function for generic SPI transmission
 * @param txBuff Pointer towards array of bytes of data to be sent
 * @param rxBuff Pointer towards array where SPI responses are stored
 * @param length Length, in bytes, of total SPI transmission
 * @return Returns if the transmission was successfully completed
 */
bool Spi_Transfer(uint8_t *txBuff, uint8_t *rxBuff, size_t length, size_t rx_length)
{
    nrfx_spim_xfer_desc_t xferDesc = NRFX_SPIM_XFER_TRX(txBuff, length, rxBuff, length);

    // Reset rx buffer and transfer flag
    memset(rxBuff, 0, length);
    spi_xfer_done = false;

    // TODO: use nonblocking mode
    ret_code_t ret = nrfx_spim_xfer(&m_spi, &xferDesc, 0);
    
    while (!spi_xfer_done) 
    {
        __WFE();
    }

    return ret == NRF_SUCCESS;
}

/**
 * @brief Function to initialize the SPI peripheral driver.
 */
void Spi_Init(void)
{
    nrfx_spim_config_t spi_config = NRFX_SPIM_DEFAULT_CONFIG;
    spi_config.frequency = NRF_SPIM_FREQ_250K;      

    spi_config.ss_pin    = AFE_CS;    
    spi_config.miso_pin  = SPI_MISO;
    spi_config.mosi_pin  = SPI_MOSI;
    spi_config.sck_pin   = SPI_CLK;
    spi_config.ss_active_high = false;
    spi_config.mode = NRF_SPIM_MODE_0;

    // TODO: supply event handler to allow nonblocking mode
    ret_code_t err_code = nrfx_spim_init(&m_spi, &spi_config, spim_event_handler, NULL);
    APP_ERROR_CHECK(err_code);
}
